﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using System.Reflection;
using ASR_023B_SDK;
using System.Configuration;

namespace ASR_023B
{
    public partial class Form2 : Form
    {
        List<BluetoothDevice> DeviceList = new List<BluetoothDevice>();
        BluetoothDevice ConnectedDevice = null;
        bool IsDisonnectBtnClick = false;

        string SavedBluetoothType = "";
        string SavedAsFingerID = "";

        /// <summary>
        /// 连接状态：0：未连接 1：连接中 2：断开中 3.已连接
        /// </summary>
        int ConnectFlag
        {
            get;
            set;
        }
        AsFingerManager asFingerManager;
        AsFinger asFinger;

        public Form2()
        {
            InitializeComponent();

            asFingerManager = AsFingerManager.SharedAsFingerManager();
            asFinger = AsFinger.SharedAsFinger();
            asFingerManager.whenReceivedBluetoothDevice += WhenReceivedBluetoothDevice;
        }
        /// <summary>
        /// 检索到设备的处理
        /// </summary>
        /// <param name="device">蓝牙设备</param>
        private void WhenReceivedBluetoothDevice(List<BluetoothDevice> devices)
        {
            this.Invoke(
                new Action(
                    delegate
                    {
                        Types.BluetoothType type = rbSpp.Checked ? Types.BluetoothType.BluetoothSPP : Types.BluetoothType.BluetoothBLE;
                        //当返回到的检索到的设备的蓝牙模式和当前画面选中的蓝牙模式不一致的场合，不做操作
                        if (asFingerManager.GetSearchBluetoothType() != type) return;
                        lvDevices.Items.Clear();
                        DeviceList = devices;
                        foreach (var device in devices)
                        {
                            ListViewItem item = new ListViewItem();
                            item.Tag = device.DeviceID;
                            item.Text = device.DeviceName;
                            lvDevices.Items.Add(item);
                        }

                        Console.WriteLine("bbbbbbbSavedAsFingerID" + SavedAsFingerID);
                        if (DeviceList.Exists(d => d.DeviceID == SavedAsFingerID) && ConnectedDevice == null)
                        {
                            Console.WriteLine("SavedAsFingerID" + SavedAsFingerID);
                            btncon_Click(null, null);
                        }
                    }
                    )
                );
        }
        /// <summary>
        /// 检索设备
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSearch_Click(object sender, EventArgs e)
        {
            if ("Search" == btnSearch.Text)
            {
                Types.BluetoothType bluetoothType;
                if (rbBLE.Checked)
                {
                    bluetoothType = Types.BluetoothType.BluetoothBLE;
                }
                else
                {
                    bluetoothType = Types.BluetoothType.BluetoothSPP;
                }
                asFingerManager.StartSearching(bluetoothType);
                lvDevices.Items.Clear();
                DeviceList.Clear();
                lblSearch.Text = "Searching...";
                btnSearch.Text = "Stop Search";
            }
            else
            {
                btnSearch.Text = "Search";
                lblSearch.Text = "Searched devices";
                //手动点击画面停止检索按钮后，取消自动连接功能
                if (SavedAsFingerID != "")
                {
                    SavedAsFingerID = "";
                    SavedBluetoothType = "";
                }
                if (asFingerManager != null) asFingerManager.StopSearching();
            }

        }
        /// <summary>
        /// 接收到条形码的处理
        /// </summary>
        /// <param name="data">条形码数据</param>
        private void WhenReceiveBarcodeData(byte[] data)
        {

            char[] chr = new char[] { '\r', '\n' };
            string value;
            Encoding encoding;

            if (rbASCII.Checked)
            {
                encoding = Encoding.ASCII;
                value = encoding.GetString(data).TrimEnd(chr);
            }
            else if (rbUTF8.Checked)
            {
                encoding = Encoding.UTF8;
                value = encoding.GetString(data).TrimEnd(chr);
            }
            else
            {
                encoding = Encoding.GetEncoding("Shift-JIS");
                value = encoding.GetString(data).TrimEnd(chr);
            }

            if (value != "")
            {
                this.Invoke(
                    new Action(
                        delegate {
                            int index = this.gvData.Rows.Add();
                            this.gvData.Rows[index].Cells[0].Value = value;
                            this.gvData.ClearSelection();
                            this.gvData.FirstDisplayedScrollingRowIndex = this.gvData.Rows.Count - 1;
                        }
                        )
                    );
            }
        }
        /// <summary>
        /// 接收到电量的处理
        /// </summary>
        /// <param name="battery">电量</param>
        private void WhenReceivedBattery(int battery)
        {
            this.Invoke(
                new Action(
                    delegate {
                        if (battery == 0)
                        {
                            picBattery.Image = imgBattery.Images[0];
                        }
                        else if (battery > 0 && battery <= 25)
                        {
                            picBattery.Image = imgBattery.Images[1];
                        }
                        else if (battery > 25 && battery <= 50)
                        {
                            picBattery.Image = imgBattery.Images[2];
                        }
                        else if (battery > 50 && battery <= 75)
                        {
                            picBattery.Image = imgBattery.Images[3];
                        }
                        else if (battery > 75 && battery <= 100)
                        {
                            picBattery.Image = imgBattery.Images[4];
                        }
                        else
                        {
                            picBattery.Image = imgBattery.Images[0];
                        }
                    }
                    )
                );
        }
        /// <summary>
        /// 接受到固件版本号的处理
        /// </summary>
        /// <param name="ver">固件版本号</param>
        private void WhenReceivedFirmwareVersion(string ver)
        {
            this.Invoke(
                new Action(
                    delegate {
                        txtFingerVer.Text = ver;
                        asFinger.GetBattery();
                        timeBattery.Enabled = true;
                    }
                    )
                );
        }
        /// <summary>
        /// 接收到数据的处理
        /// </summary>
        /// <param name="data">返回的数据</param>
        private void WhenReceivedData(byte[] data)
        {
            Console.WriteLine("Form: " + System.Text.Encoding.ASCII.GetString(data));
        }
        /// <summary>
        /// 连接状态返回的处理
        /// </summary>
        /// <param name="isConnected">true：已连接 false：未连接</param>
        private void WhenAsFingerIsConnected(bool isConnected)
        {
            if (isConnected)
            {
                this.Invoke(
                        new Action(
                            delegate {
                                asFinger.GetFirmwareVersion();
                                this.lblConnectedDev.Text = asFinger.GetBluetoothDevice().DeviceName;
                                this.lblDeviceId.Text = asFinger.GetBluetoothDevice().DeviceID;
                                ConnectedDevice = asFinger.GetBluetoothDevice();
                                if (ConnectedDevice.DeviceID == SavedAsFingerID)
                                {
                                    SavedAsFingerID = "";
                                    SavedBluetoothType = "";
                                }
                                txtSDKVer.Text = asFinger.GetSDKVersion();

                                this.btnCon.Text = "Disconnect";
                                this.btnCon.Enabled = true;
                                lblSearch.Text = "Searched devices";
                                btnSearch.Text = "Search";
                                ConnectFlag = 3;
                                gpData.Enabled = true;

                                rbBLE.Checked = ConnectedDevice.DeviceBluetoothType == Types.BluetoothType.BluetoothBLE;
                                rbSpp.Checked = ConnectedDevice.DeviceBluetoothType == Types.BluetoothType.BluetoothSPP;

                                IsDisonnectBtnClick = false;
                                timeBattery.Enabled = true;
                            })
                        );
            }

            if (!isConnected)
            {
                this.Invoke(
                        new Action(
                            delegate {
                                ConnectedDevice = null;
                                this.btnCon.Enabled = true;
                                ConnectFlag = 0;
                                btnCon.Text = "Connect";
                                lblSearch.Text = "Searched devices";
                                btnSearch.Text = "Search";
                                gpData.Enabled = false;
                                txtFingerVer.Text = "";
                                txtSDKVer.Text = "";
                                picBattery.Image = null;
                                timeBattery.Enabled = false;
                                timerConnect.Enabled = true;
                            })
                        );
            }
        }

        /// <summary>
        /// 连接
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private async void btncon_Click(object sender, EventArgs e)
        {
            string selected = "";
            if (ConnectFlag != 3 && this.lvDevices.SelectedItems.Count > 0)
            {
                ListViewItem row = this.lvDevices.SelectedItems[0];
                if (row != null && lblDeviceId.Text != row.Tag.ToString())
                {
                    selected = row.Tag.ToString();
                }
            }
            //手动点击画面连接按钮后，取消自动连接功能
            if (selected != "" && SavedAsFingerID != "" && SavedAsFingerID != selected)
            {
                SavedAsFingerID = "";
                SavedBluetoothType = "";
            }
            //设备连接有返回的状态值（并且是没有连接成功的场合）或者自动连接中可以点击连接按钮（防止自动连接中不能手动连接其他设备）
            Types.BluetoothType type = rbBLE.Checked ? Types.BluetoothType.BluetoothBLE : Types.BluetoothType.BluetoothSPP;
            //if (ConnectFlag == 0 ||
            //    (timerConnect.Enabled && ConnectFlag == 1) ||
            //    (ConnectFlag != 3 && SavedAsFingerID != "") ||
            //    (ConnectFlag != 3 && ConnectedDevice != null && selected != ConnectedDevice.DeviceID && type != ConnectedDevice.DeviceBluetoothType))
            if (ConnectFlag != 2 && ConnectFlag != 3)
            {
                ConnectFlag = 1;
                timerConnect.Enabled = false;
                string deviceID = "";

                if (selected != "")
                {
                    deviceID = selected;
                }
                else if (SavedAsFingerID != "")
                {
                    deviceID = SavedAsFingerID;
                }

                if (deviceID == "" && lblDeviceId.Text != "")
                {
                    deviceID = lblDeviceId.Text;
                }
                if (deviceID != "")
                {
                    asFinger.whenAsFingerIsConnected = WhenAsFingerIsConnected;
                    asFinger.whenReceivedBarcodeData = WhenReceiveBarcodeData;
                    asFinger.whenReceivedBattery = WhenReceivedBattery;
                    asFinger.whenReceivedData = WhenReceivedData;
                    asFinger.whenReceivedFirmwareVersion = WhenReceivedFirmwareVersion;
                    BluetoothDevice conDev = null;

                    if (DeviceList.Exists(d => d.DeviceID == deviceID))
                    {
                        conDev = DeviceList.Find(d => d.DeviceID == deviceID);
                    }
                    else if (ConnectedDevice != null)
                    {
                        conDev = ConnectedDevice;
                    }

                    if (conDev != null)
                    {
                        ConnectedDevice = conDev;
                        ConnectFlag = 1;

                        asFinger.Connect(conDev);
                        if (SavedAsFingerID != ConnectedDevice.DeviceID)
                        {
                            SavedAsFingerID = "";
                            SavedBluetoothType = "";
                        }
                    }
                    else
                    {
                        ConnectFlag = 0;
                    }
                }
            }
            else if (ConnectFlag == 3 || ConnectFlag == 2)
            {
                ConnectFlag = 2;
                IsDisonnectBtnClick = true;
                if (asFinger != null) asFinger.Disconnect();
            }
        }
        /// <summary>
        /// 开始扫描条形码
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnStart_Click(object sender, EventArgs e)
        {
            if (asFinger != null) asFinger.StartScan();
        }

        private void Form2_Load(object sender, EventArgs e)
        {
            lblDeviceId.Text = "";
            picBattery.Image = null;
            //datagridview:防止数据闪烁
            Type type = gvData.GetType();
            PropertyInfo pi = type.GetProperty("DoubleBuffered",
                BindingFlags.Instance | BindingFlags.NonPublic);
            pi.SetValue(gvData, true, null);

            gpData.Enabled = false;
            txtFingerVer.Text = "";
            txtSDKVer.Text = "";
            timerConnect.Enabled = true;

            SavedBluetoothType = ConfigurationManager.AppSettings["BluetoothType"];
            SavedAsFingerID = ConfigurationManager.AppSettings["AsFingerID"];

            if (SavedBluetoothType != null && SavedBluetoothType.Length > 0 && SavedAsFingerID != null && SavedAsFingerID.Length > 0)
            {
                rbSpp.Checked = SavedBluetoothType == "BluetoothSPP";
                rbBLE.Checked = SavedBluetoothType == "BluetoothBLE";
                btnSearch_Click(null, null);
            }
        }

        private void btnClear_Click(object sender, EventArgs e)
        {
            lvData.Items.Clear();
            gvData.Rows.Clear();
        }
        /// <summary>
        /// 停止扫描条形码
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnStopScan_Click(object sender, EventArgs e)
        {
            if (asFinger != null) asFinger.StopScan();
        }
        /// <summary>
        /// 每10秒获取一次电量
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timeBattery_Tick(object sender, EventArgs e)
        {
            if (asFinger != null)
            {
                asFinger.GetBattery();
                if (txtFingerVer.Text == "") {
                    asFinger.GetFirmwareVersion();
                }
            }
        }
        /// <summary>
        /// 蓝牙模式切换的处理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void rbType_CheckedChanged(object sender, EventArgs e)
        {
            Types.BluetoothType type = rbSpp.Checked ? Types.BluetoothType.BluetoothSPP : Types.BluetoothType.BluetoothBLE;

            if (btnSearch.Text != "Search" || btnCon.Text != "Connect")
            {
                if (asFingerManager.GetSearchBluetoothType() == Types.BluetoothType.BluetoothSPP)
                {
                    rbSpp.Checked = true;
                }
                if (asFingerManager.GetSearchBluetoothType() == Types.BluetoothType.BluetoothBLE)
                {
                    rbBLE.Checked = true;
                }
            }

            //重新获取当前蓝牙模式
            type = rbSpp.Checked ? Types.BluetoothType.BluetoothSPP : Types.BluetoothType.BluetoothBLE;
            if (type != asFingerManager.GetSearchBluetoothType())
            {
                //切换蓝牙模式的场合，停用自动连接设备的功能
                if (timerConnect.Enabled) timerConnect.Enabled = false;
                lblConnectedDev.Text = "";
                lblDeviceId.Text = "";
                lvDevices.Items.Clear();
                DeviceList.Clear();
            }
        }

        private void Form2_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (ConnectedDevice != null)
            {
                Configuration cfa = ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None);
                if (cfa.AppSettings.Settings.Count > 0)
                {
                    cfa.AppSettings.Settings["BluetoothType"].Value = Enum.GetName(typeof(Types.BluetoothType), ConnectedDevice.DeviceBluetoothType);
                    cfa.AppSettings.Settings["AsFingerID"].Value = ConnectedDevice.DeviceID;
                    cfa.Save();
                }
            }
        }

        private void timerConnect_Tick(object sender, EventArgs e)
        {
            if (ConnectFlag == 0 && IsDisonnectBtnClick == false && lblDeviceId.Text != "" && DeviceList.Exists(d => d.DeviceID == lblDeviceId.Text))
            {
                asFinger.whenAsFingerIsConnected = WhenAsFingerIsConnected;
                asFinger.whenReceivedBarcodeData = WhenReceiveBarcodeData;
                asFinger.whenReceivedBattery = WhenReceivedBattery;
                asFinger.whenReceivedData = WhenReceivedData;
                asFinger.whenReceivedFirmwareVersion = WhenReceivedFirmwareVersion;

                ConnectedDevice = DeviceList.Find(d => d.DeviceID == lblDeviceId.Text);
                asFinger.Connect(ConnectedDevice);
                ConnectFlag = 1;
            }
        }
    }
}
