﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.IO;

namespace ASR_023B
{
    public class Log
    {
        #region 列挙体
        /// <summary>ログモード</summary>
        public enum LogMode
        {
            Debug,
            Info,
            Warning,
            Error
        }

        /// <summary>エンコードモード</summary>
        public enum EncodeMode
        {
            SJIS,
            UTF8
        }
        #endregion

        #region 変数
        /// <summary>エンコードモード</summary>
        private static EncodeMode encodeMode;

        /// <summary>ログレベル</summary>
        private static LogMode logLevel;

        /// <summary>ログパス</summary>
        private static string logPath = "";

        /// <summary>ログフォルダ名</summary>
        private static string logFolder = System.AppDomain.CurrentDomain.BaseDirectory + "Log";

        /// <summary>ログファイル名</summary>
        private static string logFile = "Log";
        #endregion

        #region プロパティ
        /// <summary>エンコード</summary>
        public static EncodeMode Encode
        {
            get { return encodeMode; }
            set { encodeMode = value; }
        }

        /// <summary>ログレベル</summary>
        public static LogMode LogLevel
        {
            get { return logLevel; }
            set { logLevel = value; }
        }

        /// <summary>ログフォルダ名</summary>
        public static string LogFolder
        {
            get { return logFolder; }
            set { logFolder = value; }
        }

        /// <summary>ログファイル名</summary>
        public static string LogFile
        {
            get { return logFile; }
            set { logFile = value; }
        }
        #endregion

        #region メソッド
        /// <summary>
        /// debugログ
        /// </summary>
        /// <param name="msg"></param>
        /// <param name="ex"></param>
        public static void Debug(String msg)
        {
            WriteLog(LogMode.Debug, msg);
        }

        /// <summary>
        /// infoログ
        /// </summary>
        /// <param name="msg"></param>
        /// <param name="ex"></param>
        public static void Info(String msg, Exception ex = null)
        {
            WriteLog(LogMode.Info, msg);
        }

        /// <summary>
        /// Warningログ
        /// </summary>
        /// <param name="msg"></param>
        /// <param name="ex"></param>
        public static void Warning(String msg)
        {
            WriteLog(LogMode.Warning, msg);
        }

        /// <summary>
        /// Errorログ
        /// </summary>
        /// <param name="msg"></param>
        /// <param name="ex"></param>
        public static void Error(String msg, Exception ex = null)
        {
            WriteLog(LogMode.Error, msg, ex);
        }

        /// <summary>
        /// ログ出力
        /// </summary>
        /// <param name="msg">メッセージ</param>
        /// <param name="ex">Exception(無指定の場合はメッセージのみ出力)</param>
        /// <remarks></remarks>
        private static void WriteLog(LogMode logMode, String msg, Exception ex = null)
        {
            bool isAppend = false;

            try
            {
                if ((int)logLevel <= (int)logMode)
                {
                    // ログフォルダ名作成
                    DateTime dt = DateTime.Now;

                    // ログフォルダ名作成
                    Directory.CreateDirectory(logFolder);

                    // ログファイル名作成
                    logPath = logFolder + "\\" + logFile + dt.ToString("dd") + ".log";

                    if (File.Exists(logPath))
                    {
                        if (File.GetLastWriteTime(logPath).ToString("yyyyMMdd") == dt.ToString("yyyyMMdd"))
                        {
                            isAppend = true;
                        }
                        else
                        {
                            isAppend = false;
                        }
                    }

                    // ログ出力文字列作成
                    String logStr;
                    string modeStr = "";

                    switch (logMode)
                    {
                        case LogMode.Debug:
                            modeStr = "[Debug]";
                            break;

                        case LogMode.Info:
                            modeStr = "[Info]";
                            break;

                        case LogMode.Warning:
                            modeStr = "[Warning]";
                            break;

                        case LogMode.Error:
                            modeStr = "[Error]";
                            break;
                    }

                    logStr = dt.ToString("yyyy/MM/dd HH:mm:ss") + "\t" + modeStr + msg;
                    if (ex != null)
                    {
                        logStr = logStr + Environment.NewLine + ex.ToString();
                    }

                    // ログ出力
                    StreamWriter sw = null;
                    try
                    {
                        Encoding enc = null;

                        switch (encodeMode)
                        {
                            case EncodeMode.SJIS:
                                enc = System.Text.Encoding.GetEncoding("Shift-JIS");
                                break;

                            case EncodeMode.UTF8:
                                enc = System.Text.Encoding.UTF8;
                                break;
                        }
                        sw = new StreamWriter(logPath, isAppend, enc);
                        sw.WriteLine(logStr);
                    }
                    catch
                    {

                    }
                    finally
                    {
                        if (sw != null) sw.Close();
                    }
                }
            }
            catch { }
        }
    }
    #endregion
}
